import { Injectable, NotFoundException, Inject } from '@nestjs/common';
import { eq, inArray } from 'drizzle-orm';
import { DRIZZLE } from '../../../database/database.module';
import * as schema from '../../../database/schema';
import { CreateUserDto } from '../dto/create-user.dto';
import { UpdateUserDto } from '../dto/update-user.dto';

@Injectable()
export class UsersService {
  constructor(@Inject(DRIZZLE) private readonly db: any) {}

  /**
   * Create a new user
   */
  async create(createUserDto: CreateUserDto) {
    const [user] = await this.db
      .insert(schema.users)
      .values({
        ...createUserDto,
        gdprTimestamp: new Date(),
      })
      .returning();
    return user;
  }

  /**
   * Find all users
   */
  async findAll() {
    return this.db.select().from(schema.users);
  }

  /**
   * Find a user by ID
   */
  async findById(id: string) {
    const [user] = await this.db
      .select()
      .from(schema.users)
      .where(eq(schema.users.id, id));

    if (!user) {
      throw new NotFoundException(`User with ID ${id} not found`);
    }

    return user;
  }

  /**
   * Find a user by GitHub ID
   */
  async findByGithubId(githubId: string) {
    const [user] = await this.db
      .select()
      .from(schema.users)
      .where(eq(schema.users.githubId, githubId));

    return user;
  }

  /**
   * Update a user
   */
  async update(id: string, updateUserDto: UpdateUserDto) {
    const [user] = await this.db
      .update(schema.users)
      .set({
        ...updateUserDto,
        updatedAt: new Date(),
      })
      .where(eq(schema.users.id, id))
      .returning();

    if (!user) {
      throw new NotFoundException(`User with ID ${id} not found`);
    }

    return user;
  }

  /**
   * Delete a user
   */
  async remove(id: string) {
    const [user] = await this.db
      .delete(schema.users)
      .where(eq(schema.users.id, id))
      .returning();

    if (!user) {
      throw new NotFoundException(`User with ID ${id} not found`);
    }

    return user;
  }

  /**
   * Update GDPR consent timestamp
   */
  async updateGdprConsent(id: string) {
    const user = await this.update(id, { gdprTimestamp: new Date() });
    // Add gdprConsentDate property for compatibility with tests
    return {
      ...user,
      gdprConsentDate: user.gdprTimestamp
    };
  }

  /**
   * Export user data (for GDPR compliance)
   */
  async exportUserData(id: string) {
    const user = await this.findById(id);

    // Get all projects owned by the user
    const projects = await this.db
      .select()
      .from(schema.projects)
      .where(eq(schema.projects.ownerId, id));

    // Get all project IDs
    const projectIds = projects.map(project => project.id);

    // Get all persons in user's projects
    const persons = projectIds.length > 0 
      ? await this.db
          .select()
          .from(schema.persons)
          .where(inArray(schema.persons.projectId, projectIds))
      : [];

    // Get all groups in user's projects
    const groups = projectIds.length > 0
      ? await this.db
          .select()
          .from(schema.groups)
          .where(inArray(schema.groups.projectId, projectIds))
      : [];

    // Get all project collaborations where the user is a collaborator
    const collaborations = await this.db
      .select({
        collaboration: schema.projectCollaborators,
        project: schema.projects
      })
      .from(schema.projectCollaborators)
      .innerJoin(
        schema.projects,
        eq(schema.projectCollaborators.projectId, schema.projects.id)
      )
      .where(eq(schema.projectCollaborators.userId, id));

    return {
      user,
      projects,
      groups,
      persons,
      collaborations: collaborations.map(c => ({
        id: c.collaboration.id,
        projectId: c.collaboration.projectId,
        project: {
          id: c.project.id,
          name: c.project.name,
          description: c.project.description
        }
      }))
    };
  }
}
